/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2023                                                     **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBELIB_POP_WALL_TIME_TEST_H
#define CUBELIB_POP_WALL_TIME_TEST_H

#include <string>
#include "PerformanceTest.h"


namespace popcalculation
{
class POPWallTimeTest : public PerformanceTest
{
private:
    cube::Metric* pop_walltime;
    double        maxValue;


protected:

    void
    applyCnode( const cube::list_of_cnodes& cnodes,
                const bool                  direct_calculation = false   );

    virtual
    const std::string&
    getCommentText() const;

    inline
    virtual
    std::string
    getHelpUrl() const
    {
        std::string active_text =
            "WallClock time metric is used to control the quality of the measurement and \n"
            "it displays minimal, average, and maximal runtime across the system tree.\n"
            "By inspecting this metric the user has an opportunity to estimate weather \n"
            "the measurement is made properly to be suitable for the POP analysis.\n"
            "If the measurement is very skewed (outliners, too wide range in the times from CPU to CPU, and similar)\n"
            "most likely is this measurement has been made errorneously and is not suitable for the analysis. \n"
            "In this case, it is advisable to redo the measurement.";
        std::string not_active_text =
            "WallClock time metric is available for every Score-P/Scalasca measurement, where Time metric is present. \n"
            "If Cube Report was produced by another tool than Score-P/Scalasca, it might have missing metric Time and hence Computation Time .\n"
            "In this case, this metric reported as NAN.";

        return active_text;
    }

    virtual
    void
    adjustForTest( cube::CubeProxy* ) const;

public:
    POPWallTimeTest( cube::CubeProxy* );

    virtual
    ~POPWallTimeTest()
    {
    };
// ------ overview tests ---------

    virtual
    inline
    double
    getMaximum() const
    {
        return maxValue;
    }

    virtual
    inline
    bool
    isPercent() const
    {
        return false;
    }

    bool
    isActive() const;

    bool
    isIssue() const;

    virtual
    inline
    bool
    isNormalized() const // returns true, if range goes from 0 to 1. false -> otherwise
    {
        return false;
    }
};
};
#endif // POP_ipc_TEST_H
